img.shadow {
    box-shadow: 3px 3px 8px #222;
}

/* artist view (artist.pt) */
#artist-header {
    padding-top: 5px;
    padding-bottom: 5px;
}

/* album view (album.pt) */
.track-artists {
    font-size: 70%;
}

.albums-box {
    padding-top: 10px;
    padding-bottom: 10px;
}

.albums-type-checkbox {
    padding-right: 10px;
}
